import datergf.*;       // import DateTimeRGF package

/**
 * Demonstrating the DateTimeRGF-classes, using the documented examples.
 *
 * @author Rony G. Flatscher, 2001-03-30
 * @version 0.9
 */

class ShowDateTimeRGF{


  public static void main (String args[])
  {
     System.out.println("Testing the DateTimeRGF-classes:\n");
     test_DateRGF       ();
     test_TimeRGF       ();
     test_DateTimeRGF   ();
     test_DateFormatRGF ();
     test_NewDT();

     System.out.print("\n\t\t---The End---\n\n");
  }




  private static void test_DateRGF()
  {
      System.out.print("Testing DateRGF-class:\n\n");

      System.out.println("   Global settings:\n");
      DateRGF gregStartDate=DateRGF.getGregorianChange();
      System.out.println("\tDateRGF.sGC          ="+ gregStartDate +" (start/first day of Gregorian calendar)");
      int weekSt = gregStartDate.get(DTC.WEEK_START);  // get global setting

      System.out.println("\tDateRGF.weekStart    ="+ weekSt +" (" + DateRGF.getString(DTC.DN, weekSt)+")");
      System.out.println("\tDateRGF.dateSeparator=\""+DateRGF.dateSeparator+"\"" );

      {
         String tmp="YMD";
         if      (DateRGF.getStatic(DTC.DATE_ORDER)==DTC.MDY) tmp="MDY";
         else if (DateRGF.getStatic(DTC.DATE_ORDER)==DTC.DMY) tmp="DMY";
         System.out.println("\tDateRGF.dateOrder    ="+ tmp);
      }



      System.out.print("\n\t\t\t---\n");

      // -----------------------------------------------------------------------------------
      {
          DateRGF firstDate=new DateRGF(   1, 1, 1), // yields: '0001-01-01'
                  lastDate =new DateRGF(9999,12,31); // yields: '9999-12-31'

          int     days=lastDate.subtract(firstDate); // yields: 3652060

       System.out.println("\tlastDate  ("+lastDate+") - firstDate ("+firstDate+") \tafter: "+days+ " days");


          DateRGF tmpDate=(DateRGF)firstDate.clone();// yields: '0001-01-01'
          tmpDate.add(days);                         // yields: '9999-12-31'

       System.out.println("\tfirstDate ("+firstDate+") + "+days+" days \tafter: "+tmpDate);

       tmpDate.assign(lastDate).add(-days);
       System.out.println("\tlastDate  ("+lastDate+") - "+days+" days \tafter: "+tmpDate);
       System.out.println();



                         // get Labor Monday in 2001
          tmpDate=new DateRGF(2001,8,31);            // yields: '2001-08-31', a Friday
       System.out.println("\ttmpDate="+tmpDate);
          tmpDate.set(DTC.WEEKDAY, 1);                   // yields: '2001-09-03'
       System.out.println("\ttmpDate="+tmpDate+ "\t(Labor Day)");
       System.out.println();


          DateRGF d=DateRGF.valueOf(DTC.ENCODED_AS_INTEGER, 20190521);       // yields: '2019-05-21'
       System.out.println("\td="+d);

          d.set(DTC.WB, 0);              // yields: '2019-05-19', if weekStart=7 (Sunday)
       System.out.println("\td="+d+"\tafter setting with 'WB', i.e. 'week begin'");
          int iso_day=d.get(DTC.DOW);    // yields: 7 (ISO-number for Sunday)
       System.out.println("\tiso_day="+iso_day);

          d.set(DTC.WB, 3);              // yields: '2019-05-22', if weekStart=7 (Sunday)
       System.out.println("\td="+d+"\tafter setting with 'WB', 3");
          d.set(DTC.WE, 0);              // yields: '2019-05-25', if weekStart=7 (Sunday)
       System.out.println("\td="+d+"\tafter setting with 'WE', 0");

          int palm_days=d.get(DTC.EPOCH_PALM);           // yields: 35430
       System.out.println("\tpalm_days="+palm_days+" days to get from EPOCH_PALM to: "+d);
          int java_days=d.get(DTC.EPOCH_JAVA);           // yields: 18041
       System.out.println("\tjava_days="+java_days+" days to get from EPOCH_JAVA to: "+d);

          String day_name  =DateRGF.getString(DTC.DN, d.get(DTC.DOW));           // yields: 'Saturday'
       System.out.println("\tday_name  ="+day_name);
          String month_name=DateRGF.getString(DTC.MN, d.get(DTC.MONTH_FIELD));   // yields: 'May'
       System.out.println("\tmonth_name="+month_name);

          DateRGF easter=DateRGF.easter(d);        // yields: '2019-04-21'
       System.out.println("\teaster="+easter+" (in the year of date d: "+d+")");

      }
      System.out.print("\n\t\t---\n");

      // -----------------------------------------------------------------------------------
      {
           DateRGF e=DateRGF.valueOf(DTC.EPOCH_PALM, 0),         // yields: '1904-01-01'
                   m=new DateRGF(2001, 1, 1);                        // yields: '2001-01-01', the
                                                                     // new millenium
       System.out.println("\te="+e+" \tafter: 'DateRGF.valueOf(DTC.EPOCH_PALM, 0)'");
       System.out.println("\tm="+m+" start of 3rd millenium!");

           int palm_days=m.subtract(e);                             // yields: 35430
       System.out.println("\tpalm_days="+palm_days);

           DateRGF d=DateRGF.valueOf(DTC.EPOCH_PALM, palm_days); // yields: '2001-01-01'
       System.out.println("\td="+d+"\t\tafter: 'DateRGF.valueOf(DTC.EPOCH_PALM, palm_days)'");

           e.add(palm_days);                                         // yields: '2001-01-01'
       System.out.println("\te="+e+"\t\tafter: 'e.add(palm_days)'");
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {

        System.out.println("\textracting date from strings (setting YMD):\n");
           DateRGF a=DateRGF.valueOf("1989-05-21") ,  // yields: '1989-05-21'
                   b=DateRGF.valueOf("19930922"  ) ,  // yields: '1993-09-22'
                   c=DateRGF.valueOf("1782.10.15") ,  // yields: '1782-10-15'
                   d=DateRGF.valueOf("2010  02  28") ,  // yields: '2010-02-28'
                   e=DateRGF.valueOf("030201"    ) ,  // yields: '2003-02-01'
                   f=DateRGF.valueOf("123"       ) ;  // yields: null

       System.out.println("\ta="+a+"\tafter: DateRGF.valueOf(\"1989-05-21\")"  );
       System.out.println("\tb="+b+"\tafter: DateRGF.valueOf(\"19930922\"  )"  );
       System.out.println("\tc="+c+"\tafter: DateRGF.valueOf(\"1782.10.15\")"  );
       System.out.println("\td="+d+"\tafter: DateRGF.valueOf(\"2010  02  28\")"  );
       System.out.println("\te="+e+"\tafter: DateRGF.valueOf(\"030201\"    )"  );
       System.out.println("\tf="+f+"\t\tafter: DateRGF.valueOf(\"123\"       )"  );
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {

          DateRGF d=new DateRGF(2059, 5, 20);            // yields: '2059-05-20'
       System.out.println("\td="+d);
           int     this_date  =d.get(DTC.ENCODED_AS_INTEGER), // yields: 20590520
                   quarter    =d.get(DTC.Q),                  // yields: 2
                   semester   =d.get(DTC.HY),                 // yields: 1
                   dow        =d.get(DTC.DOW),                // yields: 2
                   ws         =d.get(DTC.WEEK_START),         // yields: 7
                   dow_ord    =d.get(DTC.DOW_ORDINAL),        // yields: 3
                   dim        =d.get(DTC.DAYS_IN_MONTH),      // yields: 31
                   palm_days  =d.get(DTC.EPOCH_PALM),         // yields: 56753
                   java_days  =d.get(DTC.EPOCH_JAVA);         // yields: 32646

       System.out.println("\tthis_date="+this_date);
       System.out.println("\tquarter  ="+quarter);
       System.out.println("\tsemester ="+semester);
       System.out.println("\tdow      ="+dow+" (ISO day number)");
       System.out.println("\tws       ="+ws +" (week starts on this ISO day number)");
       System.out.println("\tdow_ord  ="+dow_ord+ " (ordinal day of week)");
       System.out.println("\tdim      ="+dim+" (days in month)");
       System.out.println("\tpalm_days="+palm_days+" (days since start of EPOCH_PALM)");
       System.out.println("\tjava_days="+java_days+" (days since start of EPOCH_JAVA)");
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {

           DateRGF d=DateRGF.valueOf(20010101, 0);    // yields: '2001-01-01'
       System.out.println("\td="+d+" \tafter: DateRGF.valueOf(20010101, 0)");

           d.set(DTC.YEAR_FIELD, 2345);             // yields: '2345-01-01'
       System.out.println("\td="+d+"\tafter set YEAR_FIELD, 2345");
           d.set(DTC.MONTH_FIELD, 12);              // yields: '2345-12-01'
       System.out.println("\td="+d+"\tafter set MONTH_FIELD, 12");
           d.set(DTC.DAY_FIELD, 15);                // yields: '2345-12-15', a Saturday
       System.out.println("\td="+d+"\tafter set DAY_FIELD, 15");

           d.set(DTC.WEEKDAY, 1);                   // yields: '2345-12-17', the next Monday
       System.out.println("\td="+d+"\tafter set WEEKDAY, 1, i.e. 'set to next Monday'");

           d.set(DTC.ENCODED_AS_INTEGER, 17280228); // yields: '1728-02-28'
       System.out.println("\td="+d+"\tafter set ENCODED_AS_INTEGER, 17280228");
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {
           DateRGF d=new DateRGF();

           d.set(DTC.ENCODED_AS_INTEGER, 17280228);  // yields: '1728-02-28'
       System.out.println("\td="+d);
           d.set(DTC.WB, 0);              // yields: '1728-02-22', if weekStart=7
       System.out.println("\td="+d+"\tWB, add 0 days");
           d.set(DTC.WB, 3);              // yields: '1728-02-25', if weekStart=7
       System.out.println("\td="+d+"\tWB, add 3 days");
           d.set(DTC.WE, 0);              // yields: '1728-02-28', if weekStart=7
       System.out.println("\td="+d+"\tWE, add 0 days");

           d.set(DTC.QB, 0);              // yields: '1728-01-01'
       System.out.println("\td="+d+"\tQB, add 0 days");
           d.set(DTC.QE, 0);              // yields: '1728-03-31'
       System.out.println("\td="+d+"\tQE, add 0 days");
           d.set(DTC.QE, -21);            // yields: '1728-03-10
       System.out.println("\td="+d+"\tQE, add -21 days \tafter: 3 weeks before quarter ends");
                                      // i.e. three weeks before the end of the quarter
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {

                       // get Labor Day of 2001 (first Monday in September)
           DateRGF d=new DateRGF(2001, 8, 31),e; // yields: '2001-08-31'
       System.out.println("\td="+d);
           DateRGF.setGivenWeekday(1, d);              // yields: '2001-09-03'
       System.out.println("\td="+d+"\tsetGivenWeekday 1 ('next Monday')");

                       // get the date the summer time ends in 2006 (last Sunday in October)
           d=new DateRGF(2006, 11, 01);        // yields: '2006-11-01'

       System.out.println("\td="+d+" first November, Sunday before == end of summer time");
           e=(DateRGF)d.clone();
           DateRGF.setGivenWeekday(-7, d);             // yields: '2006-10-29'
       System.out.println("\td="+d+"\tsetGivenWeekday -7 ('previous Sunday')\n");
       System.out.println();

       System.out.println("\te="+e+" first November, Sunday before == end of summer time");
           e.set(DTC.WEEKDAY, -7);
       System.out.println("\te="+d+"\te.set(DTC.WEEKDAY, -7) == 'previous Sunday'");
       System.out.println();

                       // get last working day of this quarter
       System.out.println("\td="+d);
           d.set(DTC.QE, 1);                       // yields: '2007-01-01', first day of next quarter
       System.out.println("\td="+d+" d.set(DTC.QE, 1) \tafter: get end of quarter add one day");
           DateRGF.setGivenWeekday(-12345, d);         // yields: '2006-12-29', a Friday
       System.out.println("\td="+d+"\tsetGivenWeekday -12345 \tafter: previous working day (Mon-Fri)");

       String tmp=DateFormatRGF.format("EEEE", d);
       System.out.println("\t\t\td="+d+"\tweekday="+tmp);

      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {
           DateRGF d  =new DateRGF(2001,1,1);         // yields: '2001-01-01'
       System.out.println("\td="+d);
           String  mon=DateRGF.getString(DTC.DN, d.get(DTC.DOW));     // yields: "Monday"
       System.out.println("\tmon=\""+mon+"\" (retrieve name of day)");
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {

    String tagesnamen[]={"Montag", "Dienstag", "Mittwoch", "Donnerstag",
                                "Freitag", "Samstag", "Sonntag"};

           DateRGF.setString(DTC.DN, tagesnamen);

           DateRGF d  =new DateRGF(2001,1,1);         // yields: '2001-01-01'
       System.out.println("\td="+d);
           String  mon=DateRGF.getString(DTC.DN, d.get(DTC.DOW));     // yields: "Montag"
       System.out.println("\tmon=\""+mon+"\" (retrieve name of day in German)");


       // reset global changes!
       tagesnamen=new String[]{"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"};
       DateRGF.setString(DTC.DN, tagesnamen);

      }
      System.out.print("\n\t\t---\n");

      System.out.print("\n==================================================\n\n");
  }

  private static void test_TimeRGF()
  {
      System.out.print("Testing TimeRGF-class:\n\n");

      System.out.println("   Global settings:\n");
      System.out.println("\tTimeRGF.am_pm_string[0]=\""+TimeRGF.am_pm_string[0]+"\"" );
      System.out.println("\tTimeRGF.am_pm_string[1]=\""+TimeRGF.am_pm_string [1]+"\"" );
      System.out.println("\tTimeRGF.timeSeparator =\""+TimeRGF.timeSeparator+"\"" );
TimeRGF.showSecs=true;
      System.out.println("\tTimeRGF.showSecs       ="+TimeRGF.showSecs       );

      System.out.print("\n\t\t\t---\n");

      // -----------------------------------------------------------------------------------
      {

           TimeRGF time1, time2, time3, time4;

           time1=new TimeRGF(19,29,39);           // yields: "19:29:39"
       System.out.println("\ttime1="+time1);
           time2=new TimeRGF( 8, 8, 8);           // yields: "08:08:08"
       System.out.println("\ttime2="+time2);
       System.out.println();

           float diff=time1.subtract(time2);      // yields: "0.47327545"
       System.out.println("\tdiff="+diff+ " \tafter: time1.subtract(time2)");

           time3=TimeRGF.valueOf(diff);           // yields: "11:21:31"
       System.out.println("\ttime3="+time3+" \tafter: TimeRGF.valueOf(diff)");
           time4=TimeRGF.valueOf(-diff);          // yields: "12:38:29"
       System.out.println("\ttime4="+time4+" \tafter: TimeRGF.valueOf(-diff)");
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {

          TimeRGF t1=TimeRGF.valueOf( 0.47399306f);   // yields: "11:22:33"
       System.out.println("\tt1="+t1+" \tafter: TimeRGF.valueOf( 0.47399306f)");
          TimeRGF t2=TimeRGF.valueOf(-0.47399306f);   // yields: "12:37:27"
       System.out.println("\tt2="+t2+" \tafter: TimeRGF.valueOf(-0.47399306f)");
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {
          TimeRGF t1=TimeRGF.valueOf(DTC.ENCODED_AS_INTEGER, 112233);   // yields: "11:22:33"
       System.out.println("\tt1="+t1+" \tafter: TimeRGF.valueOf(DTC.ENCODED_AS_INTEGER, 112233)");
          TimeRGF t2=TimeRGF.valueOf(DTC.ENCODED_AS_INTEGER,-112233);   // yields: "12:37:27"
       System.out.println("\tt2="+t2+" \tafter: TimeRGF.valueOf(DTC.ENCODED_AS_INTEGER,-112233)");
       System.out.println();

          TimeRGF t3=TimeRGF.valueOf(DTC.ENCODED_AS_SECONDS,  40953);   // yields: "11:22:33"
       System.out.println("\tt3="+t3+" \tafter: TimeRGF.valueOf(DTC.ENCODED_AS_SECONDS,  40953)");
          TimeRGF t4=TimeRGF.valueOf(DTC.ENCODED_AS_SECONDS, -40953);   // yields: "12:37:27"
       System.out.println("\tt4="+t4+" \tafter: TimeRGF.valueOf(DTC.ENCODED_AS_SECONDS, -40953)");
       System.out.println();

          TimeRGF t5=TimeRGF.valueOf(DTC.ENCODED_AS_MILLIS ,  40953123);   // yields: "11:22:33.123"
       String tst=DateFormatRGF.format("HH:mm:ss.SSS", t5);
       System.out.println("\tt5="+t5+" \tafter: TimeRGF.valueOf(DTC.ENCODED_AS_MILLIS ,  40953123)");
       System.out.println("\tt5="+t5+" ==> "+tst+" \tafter: DateFormatRGF.format(\"HH:mm:ss.SSS\", t5)");

          TimeRGF t6=TimeRGF.valueOf(DTC.ENCODED_AS_MILLIS , -40953123);   // yields: "12:37:26.877"
       System.out.println("\tt6="+t6+" \tafter: TimeRGF.valueOf(DTC.ENCODED_AS_MILLIS ,  -40953123)");
              tst=DateFormatRGF.format("HH:mm:ss.SSS", t6);
       System.out.println("\tt6="+t6+" ==> "+tst+" \tafter: DateFormatRGF.format(\"HH:mm:ss.SSS\", t6)");

          TimeRGF t7=TimeRGF.valueOf(DTC.ENCODED_AS_MILLIS , 86399999);   // yields: "23:59:59.999"
       System.out.println("\tt7="+t7+" \tafter: TimeRGF.valueOf(DTC.ENCODED_AS_MILLIS ,  86399999)");
              tst=DateFormatRGF.format("HH:mm:ss.SSS", t7);
       System.out.println("\tt7="+t7+" ==> "+tst+" \tafter: DateFormatRGF.format(\"HH:mm:ss.SSS\", t7)");
      }
      System.out.print("\n\t\t---\n");


      boolean old24hour=TimeRGF.is24Hour, old_show_secs=TimeRGF.showSecs;

      // -----------------------------------------------------------------------------------
      {
       System.out.println("Retrieving time from Strings (is24hour==true; \tafter: 24hours):");
         TimeRGF.is24Hour=true;

           TimeRGF t1=TimeRGF.valueOf("00:01:02");    // yields: '00:01:02'
           TimeRGF t2=TimeRGF.valueOf("159");    // yields: '11:59:59'
           TimeRGF t3=TimeRGF.valueOf("125544");    // yields: '12:55:44'
           TimeRGF t4=TimeRGF.valueOf("17:01:59");    // yields: '17:01:59'
           TimeRGF t5=TimeRGF.valueOf("2311 ");    // yields: '23:11:22'
           TimeRGF t6=TimeRGF.valueOf("23 11 ");    // yields: '23:11:22'

       System.out.println("\tt1="+t1+ " \tafter: TimeRGF.valueOf(\"00:01:02\")" );
       System.out.println("\tt2="+t2+ " \tafter: TimeRGF.valueOf(\"159\")" );
       System.out.println("\tt3="+t3+ " \tafter: TimeRGF.valueOf(\"125544\")" );
       System.out.println("\tt4="+t4+ " \tafter: TimeRGF.valueOf(\"17:01:59\")" );
       System.out.println("\tt5="+t5+ " \tafter: TimeRGF.valueOf(\"2311 \")" );
       System.out.println("\tt6="+t6+ " \tafter: TimeRGF.valueOf(\"23 11 \")" );
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {

       System.out.println("Retrieving time from Strings (is24hour==false; \tafter: 12 hours with am/pm):");
         TimeRGF.is24Hour=false;

           TimeRGF t1=TimeRGF.valueOf("12:01:02 am"); // yields: '00:01:02'
           TimeRGF t2=TimeRGF.valueOf("11:59:59am"); // yields: '11:59:59'
           TimeRGF t3=TimeRGF.valueOf("12:55:44pm"); // yields: '12:55:44'
           TimeRGF t4=TimeRGF.valueOf("05:01:59 pm"); // yields: '17:01:59'
           TimeRGF t5=TimeRGF.valueOf("11:11  pm"); // yields: '23:11:22'

        TimeRGF.is24Hour=true;         // now show in 24 hour-mode

       System.out.println("\tt1="+t1+" \tafter: TimeRGF.valueOf(\"12:01:02 am\")");
       System.out.println("\tt2="+t2+" \tafter: TimeRGF.valueOf(\"11:59:59am\")");
       System.out.println("\tt3="+t3+" \tafter: TimeRGF.valueOf(\"12:55:44pm\")");
       System.out.println("\tt4="+t4+" \tafter: TimeRGF.valueOf(\"05:01:59 pm\")");
       System.out.println("\tt5="+t5+" \tafter: TimeRGF.valueOf(\"11:11:22  pm\")");

        TimeRGF.is24Hour=old24hour;
      }
      System.out.print("\n\t\t---\n");




      // -----------------------------------------------------------------------------------
      {
          TimeRGF t1=new TimeRGF(11,22,33);           // yields: "11:22:33"

          int     hour  =t1.get(DTC.HOUR_FIELD),          // yields: "11"
                  minute=t1.get(DTC.MINUTE_FIELD),        // yields: "22"
                  sec   =t1.get(DTC.SECOND_FIELD);        // yields: "33"

          int     a1=t1.get(DTC.ENCODED_AS_INTEGER);      // yields: "112233"

          int     a2=t1.get(DTC.ENCODED_AS_SECONDS);      // yields: "40953"
          int     a3=t1.get(DTC.RAW_MILLIS_FIELD);          // yields: "40953"

       System.out.println("\tt1    ="+t1+ " now getting values ...");
       System.out.println("\thour  ="+hour      +" \tafter: t1.get(DTC.HOUR_FIELD)");
       System.out.println("\tminute="+minute    +" \tafter: t1.get(DTC.MINUTE_FIELD)");
       System.out.println("\tsec   ="+sec       +" \tafter: t1.get(DTC.SECOND_FIELD)");
       System.out.println("\ta1    ="+a1        +" \tafter: t1.get(DTC.ENCODED_AS_INTEGER)");
       System.out.println("\ta2    ="+a2        +" \tafter: t1.get(DTC.ENCODED_AS_SECONDS)");
       System.out.println("\ta3    ="+a3        +" \tafter: t1.get(DTC.RAW_MILLIS_FIELD)");

      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {
          TimeRGF t1=new TimeRGF(23,59,59);   // yields: "23:59:59"
       System.out.println("\tt1="+t1+ " now SET-ting values...");

          t1.set(DTC.HOUR_FIELD, 11);             // yields: "11:59:59"
       System.out.println("\tt1="+t1+"\tset HOUR_FIELD, 11");
          t1.set(DTC.MINUTE_FIELD, 22);           // yields: "11:22:59"
       System.out.println("\tt1="+t1+"\tset MINUTE_FIELD, 22");
          t1.set(DTC.SECOND_FIELD, 33);           // yields: "11:22:33"
       System.out.println("\tt1="+t1+"\tset SECOND_FIELD, 33");

          t1.set(DTC.ENCODED_AS_INTEGER, 10001);  // yields: "01:00:01"
       System.out.println("\tt1="+t1+"\tset ENCODED_AS_INTEGER, 10001");

          t1.set(DTC.ENCODED_AS_SECONDS, 40953);  // yields: "11:22:33"
       System.out.println("\tt1="+t1+"\tset ENCODED_AS_SECONDS, 40953");
          t1.set(DTC.RAW_MILLIS_FIELD, 40954987);      // yields: "11:22:34"
       System.out.println("\tt1="+t1+"\tset RAW_MILLIS_FIELD, 40954987");
      }
      System.out.print("\n\t\t---\n");



      // -----------------------------------------------------------------------------------
      {
          TimeRGF t1=new TimeRGF(11,22,33);           // yields: "11:22:33"
          float   f1=t1.toFloat();                    // yields: "0.47399306"
       System.out.print("\tt1="+t1);
       System.out.println("\tt1.toFloat()="+f1);
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {
       System.out.println("\tDemoing toString() with different global settings:\n");


          TimeRGF t1=new TimeRGF(14,23,56);   // yields: "14:23:56"
       System.out.println("\tt1=14:23.56");

          TimeRGF.is24Hour=true;             // changing global setting
          TimeRGF.showSecs =true;             // changing global setting
          String  s1=t1.toString();           // yields: "14:23:56"
       System.out.println("\ts1="+s1+"  (TimeRGF.is24Hour=true, TimeRGF.showSecs=true)");

          TimeRGF.is24Hour=false;            // changing global setting
          TimeRGF.showSecs =false;            // changing global setting
          s1=t1.toString();                   // yields: "2:23 pm"
       System.out.println("\ts1="+s1+"  (TimeRGF.is24Hour=false, TimeRGF.showSecs=false)");
                // reset
          TimeRGF.is24Hour=true;             // changing global setting
          TimeRGF.showSecs =true;             // changing global setting
      }

        // reset values
      TimeRGF.is24Hour=old24hour;
      TimeRGF.showSecs=old_show_secs;

      System.out.print("\n\t\t---\n");


      System.out.print("\n==================================================\n\n");
  }

  private static void test_DateTimeRGF()
  {
      System.out.print("Testing DateTimeRGF-class:\n\n");

      // -----------------------------------------------------------------------------------
      {

           DateTimeRGF dt1, dt2;
           dt1=new DateTimeRGF(2001,1,1,0,0,0);      // yields: 2001-01-01 00:00:00
           dt2=new DateTimeRGF(2005,12,31,11,59,59,123); // yields: 2005-12-31 11:59:59.123
           int diff[];

       System.out.print(DateFormatRGF.format("\t'dt1='yyyy-MM-dd HH:mm:ss.SSS", dt1)+", ");
       System.out.println(DateFormatRGF.format("\t'dt2='yyyy-MM-dd HH:mm:ss.SSS", dt2)+"\n");


       System.out.println("\tdt2.subtract(dt1):");
           diff=dt2.subtract(dt1);

       System.out.println("\t\tdiff[0]="+diff[0]+" \t\tdays");
       System.out.println("\t\tdiff[1]="+diff[1]+" \tmilliseconds\n");


       System.out.println("\tdt1.subtract(dt2):");
           diff=dt1.subtract(dt2);
       System.out.println("\t\tdiff[0]="+diff[0]+" \t\tdays");
       System.out.println("\t\tdiff[1]="+diff[1]+" \tmilliseconds\n");

           dt2.update();                  // yields: actual date and time
       System.out.println(DateFormatRGF.format("\t'dt1='yyyy-MM-dd HH:mm:ss.SSS", dt1));
       System.out.println(DateFormatRGF.format("\t'dt2='yyyy-MM-dd HH:mm:ss.SSS", dt2)+" after update()");
       System.out.println();

       System.out.println("\tdt2.subtract(dt1):");
           diff=dt2.subtract(dt1);

       System.out.println("\t\tdiff[0]="+diff[0]+" \t\tdays");
       System.out.println("\t\tdiff[1]="+diff[1]+" \tmilliseconds\n");


       System.out.println("\tdt1.subtract(dt2):");
           diff=dt1.subtract(dt2);
       System.out.println("\t\tdiff[0]="+diff[0]+" \t\tdays");
       System.out.println("\t\tdiff[1]="+diff[1]+" \tmilliseconds\n");


      }
      System.out.print("\n\t\t---\n");



      // -----------------------------------------------------------------------------------
      {

                                // create a DateTimeRGF object with a
                                // value of: "2010-01-12 14:05:47.123"
         DateTimeRGF dt=new DateTimeRGF(2010,01,12,14,05,47,123);

       System.out.println(DateFormatRGF.format("\t'dt='yyyy-MM-dd HH:mm:ss.SSS", dt));

         dt.add(1, 64800000);    // add 1 3/4 days \tafter: 1 day, 18 hours (= 64800000 millis)
                                 // "dt" now: "2010-01-14 08:05:47.123"

       System.out.println("\t\tadding 1 3/4 days, i.e. (1 [day], 64800000 [msec])");
       System.out.println(DateFormatRGF.format("\t'dt='yyyy-MM-dd HH:mm:ss.SSS", dt));

       System.out.println("\t");
      }
      System.out.print("\n\t\t---\n");




      // -----------------------------------------------------------------------------------
      {


           DateTimeRGF dt=new DateTimeRGF(2010,1,12, 14,5,47,123);
                          // "dt" is now: "2010-01-12 14:05:47"
       System.out.println(DateFormatRGF.format("\t'dt='yyyy-MM-dd HH:mm:ss.SSS", dt));

           long secs_long=dt.toLong(DTC.EPOCH_PALM);
                          // "secs_long" is now: "3.346.149.947" seconds
                          //                     since DTC.EPOCH_PALM
       System.out.println("\tsecs_long="+secs_long+"\tafter: dt.toLong(DTC.EPOCH_PALM)");
      }
      System.out.print("\n\t\t---\n");


      // -----------------------------------------------------------------------------------
      {

                          // 3.346.149.947 seconds since DTC.EPOCH_PALM,
                          // which is exactly: "2010-01-12 14:05:47"
           long secs_long=3346149947123l;
       System.out.println("\tsecs_long="+secs_long);

           DateTimeRGF dt=DateTimeRGF.valueOf(DTC.EPOCH_PALM, secs_long);
                          // "dt" is now: "2010-01-12 14:05:47"
       System.out.println("\t\tafter: DateTimeRGF.valueOf(DTC.EPOCH_PALM, secs_long):");
       System.out.println(DateFormatRGF.format("\t'dt='yyyy-MM-dd HH:mm:ss.SSS", dt));


       System.out.println("\t");
      }
      System.out.print("\n\t\t---\n");


      System.out.print("\n==================================================\n\n");
  }

  private static void test_DateFormatRGF()
  {
      System.out.print("Testing DateFormatRGF-class:\n\n");
      // -----------------------------------------------------------------------------------
      {
           DateRGF d=new DateRGF(2029, 5, 21);            // yields: '2029-05-21'

           String  info1=DateFormatRGF.format("EEEE, yy-MMM-dd!",  d);  // yields: "Monday, 29-May-21!"         ???NEIN!!!
           String  info2=DateFormatRGF.format("%A, %y-%b-%d!",     d);  // yields: "Monday, 29-May-21!"

           String  info3=DateFormatRGF.format("'days so far: 'DDD", d); // yields: "days so far: 141"
           String  info4=DateFormatRGF.format("'days to go:  'ggg", d); // yields: "days to go:  224"

       System.out.println("\tinfo1="+info1);
       System.out.println("\tinfo2="+info2);
       System.out.println("\tinfo3="+info3);
       System.out.println("\tinfo4="+info4);
      }
      System.out.print("\n\t\t---\n");




      // -----------------------------------------------------------------------------------
      {
           DateRGF d=new DateRGF(2029, 5, 21);           // yields: '2029-05-21'

           String  info1=DateFormatRGF.format("EEEE, yy-MMM-dd!", d);   // yields: "Monday, 29-May-21!" ???NEIN!!!
           String  info2=DateFormatRGF.format("%A, %y-%b-%d!", d);      // yields: "Monday, 29-May-21!"

           String  info3=DateFormatRGF.format("'days so far: 'DDD", d); // yields: "days so far: 141"
           String  info4=DateFormatRGF.format("'days to go:  'ggg", d); // yields: "days to go:  224"

           String  info5=DateFormatRGF.format("'week #' ww", d);        // yields: "week # 21"

           String  info6=DateFormatRGF.format("'day of week:' w", d);   // yields: "day of week: 1"
           String  info7=DateFormatRGF.format("'ordinal day:' o", d);   // yields: "ordinal day: 2", if
                                                                        //          DateRGF.weekStart=7

           String  info8=DateFormatRGF.format("d. R. yyyy", d);         // yields: "21. V. 2029"

           String  info9=DateFormatRGF.format("%e. %B (yyyy), 'a' EEEE.", d); // yields: "21. May (2029), a Monday." ???NEIN!!!

       System.out.println("\tinfo1="+info1);
       System.out.println("\tinfo2="+info2);
       System.out.println("\tinfo3="+info3);
       System.out.println("\tinfo4="+info4);
       System.out.println("\tinfo5="+info5);
       System.out.println("\tinfo6="+info6);
       System.out.println("\tinfo7="+info7);
       System.out.println("\tinfo8="+info8);
       System.out.println("\tinfo9="+info9);
      System.out.print("\n==================================================\n\n");
      }
      System.out.print("\n\t\t---\n");



      // -----------------------------------------------------------------------------------
      System.out.print("Now the additions since 2001-03-06...\n\n");
      {
           System.out.print("Examples 'DateRGF.date2jdn(DateRGF d)':\n");

           DateRGF d1=new DateRGF(1582, 10,  4),        // Julian calendar
                   d2=new DateRGF(1582, 10,  15),       // Gregorian calendar
                   d3=new DateRGF(),
                   d4=new DateRGF();
           System.out.println("\td1="+d1+", d2="+d2+" (switch to Gregorian Calendar)");

           int   jdn1=DateRGF.date2jdn(d1),             // yields:
                 jdn2=DateRGF.date2jdn(d2);             // yields:

           System.out.println("\tjdn1="+jdn1+", jdn2="+jdn2+"\n");


           DateRGF.jdn2date(jdn1, d3);
           DateRGF.jdn2date(jdn2, d4);

           System.out.println("\tcreating Gregorian calendar dates from Julian day numbers:");
           System.out.println("\tjdn1("+jdn1+")-->d3="+d3+", jdn2("+jdn2+")-->d4="+d4);
           System.out.print("\n\t\t---\n\n");
      }



      {
          System.out.print("Examples 'DateRGF.date2jdn(year, month, day, calendar2use)':\n");

          int     jdn=DateRGF.date2jdn(2010,1,1,DTC.GREGORIAN); // yields: 2455198 (Gregorian calendar)
          System.out.println("\tjdn="+jdn);

          DateRGF   d=new DateRGF();

          DateRGF.jdn2date(jdn, d, DTC.GREGORIAN);           // yields: '2010-01-01' (GREGORIAN)
          System.out.println("\t  d="+d+" (GREGORIAN calendar) ");

          DateRGF.jdn2date(jdn, d, DTC.JULIAN);              // yields: '2009-12-19' (JULIAN)
          System.out.println("\t  d="+d+" (JULIAN calendar) ");

          System.out.print("\n\t\t---\n\n");
      }



      {
          System.out.print("Examples 'DateRGF.calendar(DateRGF d)':\n");

          DateRGF bkpD = DateRGF.getGregorianChange();

          DateRGF.setGregorianChange(1752, 9, 14);      // set first Gregorian calendar date
          System.out.println("\tGregorian change date set to: " +DateRGF.getGregorianChange()+"\n");

          DateRGF d1=new DateRGF(1659, 11, 17), // falls into the Julian calendar
                  d2=new DateRGF(2016,  2,  1), // falls into the Gregorian calendar
                  d3, d4;

          System.out.println("\td1="+d1+" (Julian calendar!) ");
          d3=DateRGF.easter(d1);                   // yields: '
          System.out.print("\t\teaster(d3)="+d3+" (Julian calendar Easter==orthodox Easter) \n");


          System.out.println("\td2="+d2+" (Gregorian calendar!) ");
          d4=DateRGF.easter(d2);                   // yields: '
          System.out.print("\t\teaster(d4)="+d4+" (Gregorian calendar Easter)\n\n");


          System.out.println("\tre-set Gregorian change date to: "+
                DateRGF.setGregorianChange(bkpD.get(DTC.YEAR_FIELD       ),
                                           bkpD.get(DTC.MONTH_FIELD      ),
                                           bkpD.get(DTC.DAY_FIELD        ))
                            );

          System.out.print("\n\t\t---\n\n");
      }




      {
          System.out.print("Examples 'DateRGF.calendar(DateRGF d, flag calendar type)':\n");

          DateRGF d1=new DateRGF(2016, 9, 1);   // yields:'2010-02-01'
          System.out.print("\td1="+d1+" \n");

          DateRGF d2=DateRGF.easter(d1.get(DTC.YEAR_FIELD), DTC.JULIAN);
          System.out.print("\td2="+d2+" (Orthodox Easter)\n");

          d2.jdn2date(d2.get(DTC.JDN), d2, DTC.GREGORIAN);  // yields: '...'
          System.out.print("\td2="+d2+" Gregorian calendar date for Orthodox Easter\n");


          System.out.print("\n\t\t---\n\n");
      }



      {
          System.out.print("Examples 'DateTimeRGF.update(obj)':\n");
          DateRGF     d =new DateRGF();        // yields: '0001-01-01'
          TimeRGF     t =new TimeRGF();        // yields: '00:00:00'
System.out.println("\td="+d+", t="+t);

          d.update();
          t.update();
System.out.println("\td="+d+", t="+t+" ... after d.update(), t.update()");

          System.out.print("\n\t\t---\n\n");
      }


      System.out.print("\n==================================================\n\n");
      {
         System.out.println("Localizable stuff (see delimiters, dayname) ...\n");

        // Localizable stuff
          DateTimeRGF dt = new DateTimeRGF(2033, 2, 24, 14, 6, 59);
          String    form = "EEEE, ";
          String s1, s2, s3, s4, s5, s6, s7, s8;

          java.util.Locale def_L = java.util.Locale.getDefault();

          s1=DateFormatRGF.format(form, dt)+dt.toString()+"\t(DateRGF/TimeRGF default)";

          DateFormatRGF.localize(java.util.Locale.GERMAN);
          s2=DateFormatRGF.format(form, dt)+dt.toString()+"\t(German Locale)";

          DateFormatRGF.localize(java.util.Locale.ITALIAN);
          s3=DateFormatRGF.format(form, dt)+dt.toString()+"\t(Italian Locale)";

          DateFormatRGF.localize(java.util.Locale.JAPAN);
          s4=DateFormatRGF.format(form, dt)+dt.toString()+"\t\t(Japanese Locale)";

          DateFormatRGF.localize(java.util.Locale.UK);
          s5=DateFormatRGF.format(form, dt)+dt.toString()+"\t(UK (Great Britain) Locale)";

          DateFormatRGF.localize(java.util.Locale.US);
          s6=DateFormatRGF.format(form, dt)+dt.toString()+"\t(US Locale)";

          DateFormatRGF.localize(def_L);
          s7=DateFormatRGF.format(form, dt)+dt.toString()+"\t(default Locale of this machine)";


                // reset to the defaults of DateRGF/TimeRGF, using American English for
                // day- and monthnames; needed for all subsequent Date/TimeRGF.toString()-output
          DateFormatRGF.localize(java.util.Locale.US);
          DateRGF.setStatic(DTC.DATE_ORDER, DTC.YMD);
          DateRGF.dateSeparator ='-';

          TimeRGF.is24Hour      =true;
          TimeRGF.timeSeparator =':';

          s8=DateFormatRGF.format(form, dt)+dt.toString()+"\t(resetted to DateRGF/TimeRGF default)";

          System.out.println("\ts1="+s1);
          System.out.println("\ts2="+s2);
          System.out.println("\ts3="+s3);
          System.out.println("\ts4="+s4);
          System.out.println("\ts5="+s5);
          System.out.println("\ts6="+s6);
          System.out.println();
          System.out.println("\ts7="+s7);
          System.out.println();
          System.out.println("\ts8="+s8);

          System.out.print("\n\t\t---\n\n");
      }

      System.out.print("\n==================================================\n\n");
  }







  static void test_NewDT()
  {
     System.out.println("DTC.EPOCH_PALM...");
     testNewDT_Epoch(DTC.EPOCH_PALM);

     System.out.println("DTC.EPOCH_JAVA...");
     testNewDT_Epoch(DTC.EPOCH_JAVA);
      System.out.print("\n==================================================\n\n");


     aha(new DateTimeRGF(1959, 5, 20, 0, 0, 0, 0),
         new DateTimeRGF(1959, 5, 22, 5, 5, 5, 5), 1);

     aha(new DateTimeRGF(1959, 5, 20, 5, 5, 5, 5),
         new DateTimeRGF(1959, 5, 22, 0, 0, 0, 0), 2);

  try   { Thread.sleep(1000);}
  catch ( Exception e )
        {System.out.println("exception caught, ---rgf.");};
     aha(new DateTimeRGF(1959, 5, 15, 1, 1, 1, 1),
         new DateTimeRGF(1959, 5, 20, 5, 5, 5, 5 ), 3);

     aha(new DateTimeRGF(1959, 5, 15, 5, 5, 5, 5 ),
         new DateTimeRGF(1959, 5, 20, 1, 1, 1, 1 ), 4);


     DateTimeRGF dt=new DateTimeRGF().update();
     DateTimeRGF dt2=new DateTimeRGF();
     aha(dt, new DateTimeRGF(1959, 5, 20, 1, 1, 1, 1), 5);

//     waba.sys.Vm.sleep(250);

     try   { Thread.sleep(250);}
     catch ( Exception e )
           {System.out.println("exception caught, ---rgf.");};


     dt2.update();



     System.out.print("\n\t(The following DTs differ at least by 250 +/- 30 msec)");
     aha(dt, dt2, 6);


  }



  static void testNewDT_Epoch(int epoch)
  {
     DateRGF     ed = DateRGF.setDefaultEpochDate(epoch);

System.out.println("\t... DateRGF of epoch (used for toLong() and toInt()): "+ed+"\n");

     DateTimeRGF dt = new DateTimeRGF(1959, 5, 20, 18, 59, 12, 987);
     System.out.println(DateFormatRGF.format("\t'dt ='yyyy-MM-dd HH:mm:ss.SSS", dt));
     System.out.println();

     long millis     = dt.toLong(epoch);
     DateTimeRGF dt1 = DateTimeRGF.valueOf(epoch, millis);

     System.out.println("\t\tmillis (long)="+millis);
     System.out.println(DateFormatRGF.format("\t\t'dt1='yyyy-MM-dd HH:mm:ss.SSS", dt1)+" | "+"DateTimeRGF.valueOf(epoch, millis)");
     System.out.println();


     int secs        = dt.toInt(epoch);
     DateTimeRGF dt2 = DateTimeRGF.valueOf(epoch, secs  );

     System.out.println("\t\tsecs   (int) ="+secs);
     System.out.println(DateFormatRGF.format("\t\t'dt2='yyyy-MM-dd HH:mm:ss.SSS", dt2)+" | "+"DateTimeRGF.valueOf(epoch, secs)");

     System.out.println("\n\t---   ---   ---\n\n");

  }


  static  void aha (DateTimeRGF dt1, DateTimeRGF dt2, int nr)
  {
     DateTimeRGF dt3 = (DateTimeRGF) dt1.clone(),
                 dt4 = (DateTimeRGF) dt2.clone();


     int [] diff;

     System.out.println("\n\ncalculating differences(), run # "+ nr  +"...\n\n");
     System.out.println(DateFormatRGF.format("\t'dt3='yyyy-MM-dd HH:mm:ss.SSS", dt3));
     System.out.println(DateFormatRGF.format("\t'dt4='yyyy-MM-dd HH:mm:ss.SSS", dt4));

     diff=dt4.subtract(dt3);
     System.out.println("\n\t   dt4.subtract(dt3): diff[0]="+diff[0]+" [days], diff[1]="+diff[1]+" [msec]");
     System.out.println();

     dt3.add(diff[0], diff[1]);
     System.out.println(DateFormatRGF.format("\t'dt3='yyyy-MM-dd HH:mm:ss.SSS", dt3)+"\t[after 'dt3.add(diff)' == 'dt4']");
     System.out.println("\n\t\t---\n");


     dt3=(DateTimeRGF) dt2.clone();
     dt4=(DateTimeRGF) dt1.clone();

     System.out.println(DateFormatRGF.format("\t'dt3='yyyy-MM-dd HH:mm:ss.SSS", dt3));
     System.out.println(DateFormatRGF.format("\t'dt4='yyyy-MM-dd HH:mm:ss.SSS", dt4));

     diff=dt4.subtract(dt3);
     System.out.println("\n\t   dt4.subtract(dt3): diff[0]="+diff[0]+" [days], diff[1]="+diff[1]+" [msec]");
     System.out.println();

     dt3.add(diff[0], diff[1]);
     System.out.println(DateFormatRGF.format("\t'dt3='yyyy-MM-dd HH:mm:ss.SSS", dt3)+"\t[after 'dt3.add(diff)' == 'dt4']");
     System.out.println("\n\t\t---\n");
  }






  static String ri(DateRGF aha, int len)
  {
     if (aha==null) return "<null>";
     String blank="                                    ";
     String strDate=aha.toString();
     len=len-strDate.length();
     if (len>0) return blank.substring(0,len)+strDate;
     return strDate;

  }

}


